/*
 * Copyright 2010-2014 Ning, Inc.
 * Copyright 2014-2020 Groupon, Inc
 * Copyright 2020-2021 Equinix, Inc
 * Copyright 2014-2021 The Billing Project, LLC
 *
 * The Billing Project licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */


package org.killbill.billing.client.model.gen;

import java.util.Objects;
import java.util.Arrays;
import com.fasterxml.jackson.annotation.JsonProperty;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import org.joda.time.DateTime;
import org.killbill.billing.catalog.api.BillingPeriod;
import org.killbill.billing.client.model.gen.AuditLog;
import org.killbill.billing.entitlement.api.SubscriptionEventType;

/**
 *           DO NOT EDIT !!!
 *
 * This code has been generated by the Kill Bill swagger generator.
 *  @See https://github.com/killbill/killbill-swagger-coden
 */
import org.killbill.billing.client.model.KillBillObject;

public class EventSubscription extends KillBillObject {

    private UUID eventId = null;

    private BillingPeriod billingPeriod = null;

    private DateTime effectiveDate = null;
    
    private DateTime catalogEffectiveDate = null;

    private String plan = null;

    private String product = null;

    private String priceList = null;

    private SubscriptionEventType eventType = null;

    private Boolean isBlockedBilling = null;

    private Boolean isBlockedEntitlement = null;

    private String serviceName = null;

    private String serviceStateName = null;

    private String phase = null;



    public EventSubscription() {
    }

    public EventSubscription(final UUID eventId,
                     final BillingPeriod billingPeriod,
                     final DateTime effectiveDate,
                     final DateTime catalogEffectiveDate,
                     final String plan,
                     final String product,
                     final String priceList,
                     final SubscriptionEventType eventType,
                     final Boolean isBlockedBilling,
                     final Boolean isBlockedEntitlement,
                     final String serviceName,
                     final String serviceStateName,
                     final String phase,
                     final List<AuditLog> auditLogs) {
        super(auditLogs);
        this.eventId = eventId;
        this.billingPeriod = billingPeriod;
        this.effectiveDate = effectiveDate;
        this.catalogEffectiveDate = catalogEffectiveDate;
        this.plan = plan;
        this.product = product;
        this.priceList = priceList;
        this.eventType = eventType;
        this.isBlockedBilling = isBlockedBilling;
        this.isBlockedEntitlement = isBlockedEntitlement;
        this.serviceName = serviceName;
        this.serviceStateName = serviceStateName;
        this.phase = phase;

    }


    public EventSubscription setEventId(final UUID eventId) {
        this.eventId = eventId;
        return this;
    }

    public UUID getEventId() {
        return eventId;
    }

    public EventSubscription setBillingPeriod(final BillingPeriod billingPeriod) {
        this.billingPeriod = billingPeriod;
        return this;
    }

    public BillingPeriod getBillingPeriod() {
        return billingPeriod;
    }

    public EventSubscription setEffectiveDate(final DateTime effectiveDate) {
        this.effectiveDate = effectiveDate;
        return this;
    }

    public DateTime getEffectiveDate() {
        return effectiveDate;
    }

    public DateTime getCatalogEffectiveDate() {
		return catalogEffectiveDate;
	}

	public void setCatalogEffectiveDate(DateTime catalogEffectiveDate) {
		this.catalogEffectiveDate = catalogEffectiveDate;
	}

	public EventSubscription setPlan(final String plan) {
        this.plan = plan;
        return this;
    }

    public String getPlan() {
        return plan;
    }

    public EventSubscription setProduct(final String product) {
        this.product = product;
        return this;
    }

    public String getProduct() {
        return product;
    }

    public EventSubscription setPriceList(final String priceList) {
        this.priceList = priceList;
        return this;
    }

    public String getPriceList() {
        return priceList;
    }

    public EventSubscription setEventType(final SubscriptionEventType eventType) {
        this.eventType = eventType;
        return this;
    }

    public SubscriptionEventType getEventType() {
        return eventType;
    }

    public EventSubscription setIsBlockedBilling(final Boolean isBlockedBilling) {
        this.isBlockedBilling = isBlockedBilling;
        return this;
    }

    @JsonProperty(value="isBlockedBilling")
    public Boolean isBlockedBilling() {
        return isBlockedBilling;
    }

    public EventSubscription setIsBlockedEntitlement(final Boolean isBlockedEntitlement) {
        this.isBlockedEntitlement = isBlockedEntitlement;
        return this;
    }

    @JsonProperty(value="isBlockedEntitlement")
    public Boolean isBlockedEntitlement() {
        return isBlockedEntitlement;
    }

    public EventSubscription setServiceName(final String serviceName) {
        this.serviceName = serviceName;
        return this;
    }

    public String getServiceName() {
        return serviceName;
    }

    public EventSubscription setServiceStateName(final String serviceStateName) {
        this.serviceStateName = serviceStateName;
        return this;
    }

    public String getServiceStateName() {
        return serviceStateName;
    }

    public EventSubscription setPhase(final String phase) {
        this.phase = phase;
        return this;
    }

    public String getPhase() {
        return phase;
    }

    @Override
    public boolean equals(java.lang.Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        EventSubscription eventSubscription = (EventSubscription) o;
        return Objects.equals(this.eventId, eventSubscription.eventId) &&
            Objects.equals(this.billingPeriod, eventSubscription.billingPeriod) &&
            Objects.equals(this.effectiveDate, eventSubscription.effectiveDate) &&
            Objects.equals(this.catalogEffectiveDate, eventSubscription.catalogEffectiveDate) &&
            Objects.equals(this.plan, eventSubscription.plan) &&
            Objects.equals(this.product, eventSubscription.product) &&
            Objects.equals(this.priceList, eventSubscription.priceList) &&
            Objects.equals(this.eventType, eventSubscription.eventType) &&
            Objects.equals(this.isBlockedBilling, eventSubscription.isBlockedBilling) &&
            Objects.equals(this.isBlockedEntitlement, eventSubscription.isBlockedEntitlement) &&
            Objects.equals(this.serviceName, eventSubscription.serviceName) &&
            Objects.equals(this.serviceStateName, eventSubscription.serviceStateName) &&
            Objects.equals(this.phase, eventSubscription.phase) &&
            true /* ignoring this.auditLogs for identity operations */;
    }

    @Override
    public int hashCode() {
        return Objects.hash(eventId,
            billingPeriod,
            effectiveDate,
            catalogEffectiveDate,
            plan,
            product,
            priceList,
            eventType,
            isBlockedBilling,
            isBlockedEntitlement,
            serviceName,
            serviceStateName,
            phase,
            0 /* ignoring auditLogs for identity operations */ );
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("class EventSubscription {\n");
        sb.append("    ").append(toIndentedString(super.toString())).append("\n");
        sb.append("    eventId: ").append(toIndentedString(eventId)).append("\n");
        sb.append("    billingPeriod: ").append(toIndentedString(billingPeriod)).append("\n");
        sb.append("    effectiveDate: ").append(toIndentedString(effectiveDate)).append("\n");
        sb.append("    catalogEffectiveDate: ").append(toIndentedString(catalogEffectiveDate)).append("\n");
        sb.append("    plan: ").append(toIndentedString(plan)).append("\n");
        sb.append("    product: ").append(toIndentedString(product)).append("\n");
        sb.append("    priceList: ").append(toIndentedString(priceList)).append("\n");
        sb.append("    eventType: ").append(toIndentedString(eventType)).append("\n");
        sb.append("    isBlockedBilling: ").append(toIndentedString(isBlockedBilling)).append("\n");
        sb.append("    isBlockedEntitlement: ").append(toIndentedString(isBlockedEntitlement)).append("\n");
        sb.append("    serviceName: ").append(toIndentedString(serviceName)).append("\n");
        sb.append("    serviceStateName: ").append(toIndentedString(serviceStateName)).append("\n");
        sb.append("    phase: ").append(toIndentedString(phase)).append("\n");
        sb.append("    auditLogs: ").append(toIndentedString(auditLogs)).append("\n");
        sb.append("}");
        return sb.toString();
    }

    /**
     * Convert the given object to string with each line indented by 4 spaces
     * (except the first line).
     */
    private String toIndentedString(java.lang.Object o) {
        if (o == null) {
            return "null";
        }
        return o.toString().replace("\n", "\n    ");
    }
}

